@@ -14,15 +14,16 @@
 # include "functions.h"
 #endif
 
+
 #ifdef HAVE_SIGNAL_H
 # include <signal.h>
 #endif
 
 #ifdef ENABLE_NLS
 # include <libintl.h>
-# define _(String) gettext (String) /* NLS too */
+# define _(String) gettext (String) 
 #else
-# define _(String) (String) /* NLS too */
+# define _(String) (String) 
 #endif
 
 #ifdef HAVE_UTIME_H
@@ -30,8 +31,6 @@
 #endif
 
 
-
-
 /* for open */
 #ifdef HAVE_SYS_TYPES_H
 # include <sys/types.h>
@@ -66,12 +65,14 @@
 #define SAFER64 6
 #define SAFER128 7
 #define CAST128 8
+#define TEAN 9
 
 #define ENCRYPT 0
 #define DECRYPT 1
 
 #define CBC 0
 #define ECB 1
+#define CFB 2
 
 #if SIZEOF_LONG_INT == 4
  typedef unsigned long word32;
