@@ -18,9 +18,10 @@
 
 #ifndef DEFINES_H
 # define DEFINES_H
-# include "defines.h"
+# include <defines.h>
 #endif
 
+#include <bzero.h>
 #include "extra.h"
 #include "bits.h"
 #include "locks.h"
@@ -28,66 +29,8 @@
 extern char* outfile;
 extern int stream_flag;
 
-/* expands the one keyword to three (for tripleDES)
- */
-int BreakToThree(void * key, unsigned int keylen, void * keyword1, void * keyword2, void * keyword3)
-{
-unsigned int D,y,i,j,z;
-unsigned char key_buf[5][32];  /* up to sixteen  (128/4=32) */
-char *tmpkey=key;
-
-  if (keylen<3) {
-   fprintf(stderr,_("Not enough long keyword for tripleDES (3 char at least)\n"));
-   return 1;
-  }
-  else {
-
-      if (keylen>=24) keylen=24;
-
-          D=keylen/3;
-          y=keylen % 3;
-
-          for (i=0;i<3;i++) {
-
-              Bzero(key_buf[i],sizeof(key_buf[i]));
-              if (i==2) { /* last round */
-                memmove(key_buf[i],tmpkey,D+y);
-              }
-              else {
-               memmove(key_buf[i],tmpkey,D);    
-              }
-         
-              for (j=0;j<D;j++) {
-                for (z=0;z<(keylen-j-1);z++)
-                  memmove(&tmpkey[z],&tmpkey[z+1],1);
-              }
-          }
-
-         memmove(keyword1,key_buf[0],8); 
-         memmove(keyword2,key_buf[1],8); 
-         memmove(keyword3,key_buf[2],8); 
-         return 0;
-}
-
-}
 
 
-/* Byte swap a 32bit integer */
-word32 byteswap(word32 x)
-{
-	register char *cp,tmp;
-
-	cp = (char *)&x;
-	tmp = cp[3];
-	cp[3] = cp[0];
-	cp[0] = tmp;
-
-	tmp = cp[2];
-	cp[2] = cp[1];
-	cp[1] = tmp;
-
-	return x;
-}
 
 char * my_getpass(char * prt) {
 char *atmp;
@@ -153,8 +96,10 @@ int show_mode(int ende, int stype,int smode,char *output,int bitmode)
   if (stype==ThreeWAY) fprintf (stderr,_( "Algorithm: 3-WAY\n"));
   if (stype==BLOWFISH) fprintf (stderr,_( "Algorithm: Blowfish\n"));
   if (stype==GOST) fprintf (stderr,_( "Algorithm: GOST\n"));
-  if (stype==SAFER64) fprintf (stderr,_( "Algorithm: SAFER64\n"));  
-  if (stype==SAFER128) fprintf (stderr,_( "Algorithm: SAFER128\n"));
+  if (stype==SAFER64) fprintf (stderr,_( "Algorithm: SAFER-SK64\n"));  
+  if (stype==SAFER128) fprintf (stderr,_( "Algorithm: SAFER-SK128\n"));
+  if (stype==CAST128) fprintf (stderr,_( "Algorithm: CAST-128\n"));
+  if (stype==TEAN) fprintf (stderr,_( "Algorithm: TEA (extended)\n"));  
   if (stype==CRYPT) {
     fprintf (stderr,_( "Algorithm: Unix Crypt\n"));  
   }
@@ -162,6 +107,8 @@ int show_mode(int ende, int stype,int smode,char *output,int bitmode)
 
   if (smode==CBC) fprintf (stderr,_( "Mode: CBC\n"));
   if (smode==ECB) fprintf (stderr,_( "Mode: ECB\n"));
+  if (smode==CFB) fprintf (stderr,_( "Mode: CFB\n"));
+  
   }
   
   if (bitmode==BITS7) fprintf (stderr,_( "Keyword Bit Mode: 7bits/char\n"));
