@@ -23,14 +23,16 @@
  *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#include "mcrypt.h"
-#include "blowfish.h"
-#include "bits.h"
-#include "lcrypt.h"
-#include "sunlink.h"
-#include "crc32.h"
-#include "locks.h"
-#include "crypt.h"
+#include <mcrypt.h>
+#include <swap.h>
+#include <bzero.h>
+#include <extra.h>
+#include <bits.h>
+#include <lcrypt.h>
+#include <sunlink.h>
+#include <crc32.h>
+#include <locks.h>
+#include <crypt.h>
 
  unsigned int bit7_flag=BITS7;
  unsigned int stream_flag=FALSE;
@@ -48,15 +50,14 @@ void usage(char *name)
      fprintf(stderr,_("Usage: mcrypt FILE1 FILE2 ...\n"));
      fprintf(stderr,_("Usage: mcrypt -d FILE1 FILE2 ...\n\n"));
 
-     fprintf(stderr,_("mini-crypt encrypts and decrypts files using the des, triple des \n\
-3-way, gost and blowfish algorithms.\n\
+     fprintf(stderr,_("mini-crypt encrypts/decrypts files using several symmetric block algorithms.\n\
 \n\
  -d --decrypt                 decrypts\n\
  -a --algorithm=ALGORITHM     specify the encryption and decryption algorithm.\n\
-                              Legal parameters are: 3way, des, 3des, gost,\n\
+                              Legal parameters are: 3way, des, 3des, gost, tea,\n\
                               safer64, safer128, cast128, crypt and blowfish. \n\
                               (Default is blowfish)\n\
- -m --mode=MODE               specify the mode (ecb or cbc). (Default is cbc)\n\
+ -m --mode=MODE               specify the mode (cfb, ecb or cbc). (Default is cbc)\n\
  -k --key=KEYWORD             keyword to use.\n\
  -7 --7bit                    keyword in 7bit mode. (default)\n\
  -8 --8bit                    keyword in 8bit mode.\n\
@@ -210,7 +211,20 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n"));
 		case 'm':
 			modet= malloc(strlen(optarg)+1);
 			strcpy(modet,optarg);
-			if (modet[0]=='e' || modet[0]=='E') mode=1;
+			if (strncasecmp(modet,"ECB",3)==0) {
+			  mode=ECB;
+			}
+                        else {
+                          if (strncasecmp(modet,"CBC",3)==0) {
+                            mode=CBC;
+			  }
+			  else {
+                            if (strncasecmp(modet,"CFB",3)==0) {
+                              mode=CFB;
+	  	            }			  
+			  }
+                        
+                        }
 			free(modet);
 			break;
 		case 'a':
@@ -248,9 +262,14 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n"));
                                     type=SAFER128;
                                   }
                                   else {
-                                    if (strncasecmp(typet,"CAST128",8)==0) {
+                                    if (strncasecmp(typet,"CAST128",7)==0) {
                                     type=CAST128;
                                     }
+                                    else {
+                                       if (strncasecmp(typet,"TEAN",4)==0 || strncasecmp(typet,"TEA",3)==0) {
+                                       type=TEAN;
+                                       }
+                                    }
                                   }
                                 }
                               }
@@ -400,12 +419,14 @@ for(i=0;i<file_count;i++) {
 			case SAFER64:
 			case SAFER128:
                         case CAST128:
+                        case TEAN:
 			 x=decrypt_general(type,dinfile,outfile,keyword);
 			 break;
 			case CRYPT:
 			 x=encrypt_ucrypt(dinfile,outfile,keyword);
 			 break;			 
                         default:
+                         fprintf(stderr,_("Algorithm unsupported in this version\n"));
                          x=1;
                          break;
                         }
@@ -447,7 +468,8 @@ for(i=0;i<file_count;i++) {
 	        	case GOST:
 	        	case SAFER64:
 	        	case SAFER128:
-                        case CAST128: 	        	
+                        case CAST128:
+                        case TEAN:
 	        	 x=encrypt_general(type,einfile,outfile,keyword);
 	        	 break;
 	        	 
@@ -610,7 +632,7 @@ int encrypt_general(int algorithm, char *fromfile,char *tofile,char *key)
 {
 unsigned char counter=0;
 word32 fcrc32;
-int i=0,j=0,ch=0,start=0,how=0;
+int i=0,j=0,ch=0,how=0;
 char *tmp,*copychar;
 unsigned int len=0;
 FILE *RTOF;
@@ -623,16 +645,14 @@ int td;
 word32 *IV=0;
 word32 *keyword=calloc(keysize,1); 
 word32 *ciphertext=malloc(blocksize);
-word32 *ciphertextcbc=0;
 FILE * TOF; 
 
  TOF=tmpfile();
  write_lock( fileno((FILE*)TOF));
         
 
-if (mode==CBC) {
+if (mode==CBC || mode==CFB) {
  IV=malloc(blocksize);
- ciphertextcbc=malloc(blocksize); 
  /* Initialisation Vector */
  srand(time(0));
 
@@ -706,17 +726,33 @@ if (mode==CBC) {
 
  if (quiet==FALSE) show_mode(ENCRYPT,algorithm,mode,tofile,bit7_flag);
 
- if (algorithm != BLOWFISH) {
-   len=keysize; /* Only Blowfish has non-fixed key */
- }
- else { /* blowfish */
-   if (len<2) {
+
+  if (algorithm != BLOWFISH) {
+    len=keysize; /* Only Blowfish has non-fixed key */
+  }
+  else { /* blowfish */
+    if (len<2) {
     fprintf(stderr, _("Too small key for this algorithm. Aborting.\n"));
     return 1;
     }
- }
+  }
 
- td = init_mcrypt(algorithm,keyword,len);
+
+
+  switch (mode) {
+   case ECB:
+    td = init_mcrypt_ecb(algorithm,keyword,len);
+    break;
+   case CBC:
+    td = init_mcrypt_cbc(algorithm,keyword,len);
+    break;
+   case CFB:
+    td = init_mcrypt_cfb(algorithm,keyword,len,IV);
+    break;    
+   default:
+    return 1;
+    break;
+  }  
 
  if (bare_flag==FALSE) clear_crc32();
 
@@ -726,52 +762,79 @@ if (mode==CBC) {
 switch (mode) {
 
  case CBC:
+
+  /* Put the IV into the file */
+  mcrypt_cbc(td, IV, blocksize);
+  if (fwrite(IV, 1, blocksize, TOF) == 0) {perror("fwrite"); return 1;}
+
   do
   {
-
-    /* If first time use the Init. vector */
-      if (start==0) {
-        memmove(ciphertext,IV,blocksize);
-        start=1;
-      }
-      else {
-
-     /* else read the first n bytes of the file and store to ciphertext */
+  /* read the first n bytes of the file and store to ciphertext */
  
-                   how=fread(ciphertext, 1, blocksize,FROMF);
- 	           if (how < blocksize) {
+                   how=fread(ciphertext,1,blocksize,FROMF);
 
-                      if ( how ==0 && feof(FROMF)==0) {
-                         perror("fread"); 
-                         return 1;
-                      }
-                   
-		      counter=(unsigned char) how;
-                      ch=1;
+  	           if (how < blocksize) {
+                    if ( how ==0 && feof(FROMF)==0) {
+                       perror("fread"); 
+                       return 1;
+                    }
+
+		    counter=((unsigned char) how);
+                    ch=1;
                    }
 
-       /* crc32 */
-        if (bare_flag==FALSE) crc32(ciphertext, how);
-
-       /* for cbc xor the plaintext with the old ciphertext block */
-          for(j=0; j<blocksize/sizeof(word32); j++)
-            ciphertext[j] ^= ciphertextcbc[j];
-
-      }
 
 
-     mcrypt(td, ciphertext);
+  /* crc32 */
+   if (bare_flag==FALSE) crc32(ciphertext, how);
 
-     /* for cbc keep the old ciphertext */
-     memmove(ciphertextcbc,ciphertext,blocksize);
+   mcrypt_cbc(td, ciphertext,blocksize);
 
          
-     if (fwrite(ciphertext, 1, blocksize, TOF) == 0) {perror("fwrite"); return 1;}
-      if (ch==1) if (fwrite(&counter, 1, sizeof(counter), TOF) == 0) {perror("fwrite"); return 1;}
+    if (fwrite(ciphertext, 1, blocksize, TOF) == 0) {perror("fwrite"); return 1;}
+    if (ch==1) {
+      if (fwrite(&counter, 1, sizeof(counter), TOF) == 0) {perror("fwrite"); return 1;}
+    }
+
   }
   while(ch==0);
   break;
 
+ case CFB:
+
+  /* Put the IV into the file */
+  if (fwrite(IV, 1, blocksize, TOF) == 0) {perror("fwrite"); return 1;}
+
+  do
+  {
+  /* read 1 byte of the file and store to ciphertext */
+ 
+                   how=fread(ciphertext,1, sizeof(ciphertext),FROMF);
+
+  	           if (how < 1) {
+                    if ( how ==0 && feof(FROMF)==0) {
+                       perror("fread"); 
+                       return 1;
+                    }
+
+                    ch=1;
+                   }
+
+
+
+   /* crc32 */
+    if (bare_flag==FALSE) crc32(ciphertext, how);
+    mcrypt_cfb(td, ciphertext,how);
+         
+    if (ch==0) {
+      if (fwrite(ciphertext, 1, how, TOF) == 0) {perror("fwrite"); return 1;}
+    }
+
+  }
+  while(ch==0);
+  break;
+
+
  case ECB:
   do
   {
@@ -795,7 +858,7 @@ switch (mode) {
   /* crc32 */
    if (bare_flag==FALSE) crc32(ciphertext, how);
 
-   mcrypt(td, ciphertext);
+   mcrypt_ecb(td, ciphertext, blocksize);
 
          
     if (fwrite(ciphertext, 1, blocksize, TOF) == 0) {perror("fwrite"); return 1;}
@@ -818,6 +881,8 @@ switch (mode) {
    fcrc32=byteswap(get_crc32());
 #endif   
 
+
+
 /* encrypt the crc32, but first put some random data there 
  */
    for(j=0; j<blocksize/sizeof(word32); j++)
@@ -855,16 +920,24 @@ switch (mode) {
  }
                 
     fclose(TOF);
-    end_mcrypt(td);
-  
     fclose(RTOF);
     fclose(FROMF);
 /* Ready */
 
-    if (mode==CBC) {
+
+  switch (mode) {
+   case ECB:
+     end_mcrypt_ecb(td);
+     break;
+   case CBC:
+     end_mcrypt_cbc(td);
      free(IV);
-     free(ciphertextcbc);     
-    }
+     break;
+   case CFB:
+     end_mcrypt_cfb(td);
+     free(IV);
+     break;     
+  }  
 
     Bzero(keyword, keysize);
     free(keyword); 
@@ -884,7 +957,7 @@ int decrypt_general(int algorithm, char *fromfile,char *tofile,char *key)
 {
 word32 fcrc32, newcrc32;
 unsigned char counter=0;
-int i=0,j=0,ch=0,start=0;
+int i=0,ch=0,start=0;
 char *tmp;
 int keysize=get_key_size(algorithm);
 int blocksize=get_block_size(algorithm);
@@ -971,8 +1044,21 @@ if (quiet==FALSE) show_mode(DECRYPT,algorithm,mode,tofile,bit7_flag);
     }
  }
  
- td = init_mcrypt(algorithm,keyword,len);
-
+  switch (mode) {
+    case CBC:
+      td = init_mcrypt_cbc(algorithm,keyword,len);
+      break;
+    case ECB:
+      td = init_mcrypt_ecb(algorithm,keyword,len);
+      break;
+    case CFB:
+      td = init_mcrypt_ecb(algorithm,keyword,len);
+      break;      
+    default:
+      return 1;
+      break;
+  }
+    
 /* Decrypt CRC32 stored in file */
  if (bare_flag==FALSE) {
 
@@ -992,77 +1078,9 @@ switch (mode) {
 
  case CBC:
   /* decryption starts here */
-   do
-   {
-
-    /* reads the first n bytes of the file and stores the to ciphertext */
-    /* /4 is here because word32 is 4 bytes */
-           i=fread( ciphertext, 1, blocksize, FROMF);
-
-           if (i < blocksize) {
-             if ( ferror(FROMF)!=0 ) {
-               perror("fread"); 
-               return 1;
-             }
-             else { /* End of file */
-               ch=1;
-               if (i==0) {fprintf(stderr,_("Incorrect file type\n")); return 1;}
-               memmove(&counter, ciphertext, sizeof(counter));
-               if (counter >= blocksize) {fprintf(stderr,_("Incorrect file type\n")); return 1;}
-             }
-           }
-
-         /* Decrypt the ciphertext */
-
-        memmove(ciphertext_orig,ciphertext,blocksize); /* keep backup */
-
-
-        mdecrypt(td, ciphertext);
-	
-  /* if cbc: xor the new plaintext with the old ciphertext */
-        if (start>0) {
-          for(j=0; j<blocksize/sizeof(word32); j++) {
-            ciphertext[j] ^= ciphertext_old[j];
-          }
-        }
-
-        memmove(ciphertext_old,ciphertext_orig,blocksize);
-
-
-/* Buffer isn't immediately written here. It waits a round to do
- * so we can keep track of the end of file garbage and the IV at the begining
- */
-
-         if (start>1) {
- 
-		if (ch==1) {
-                  if (counter==0) break;
-		  if (fwrite(buffer,(int)counter,1,RTOF)==0) {perror("fwrite"); return 1;}
-  		   /* CRC32 */
-          	  if (bare_flag==FALSE) crc32(buffer, (int)counter);
-
-		}
-		else {
-
-   		      if (fwrite(buffer,blocksize,1,RTOF)==0) {perror("fwrite"); return 1;}
-                       /* CRC32 */
-                      if (bare_flag==FALSE) crc32(buffer, blocksize);
-
-                }
-         }
-
-
-
-         memmove(buffer,ciphertext,blocksize);  /* copy the plaintext to buffer */
-
-         start++; /* ok enter that functions */
-
-   }
-   while(ch==0);
-   break;
-
-
- case ECB:
+   fread( ciphertext, 1, blocksize, FROMF); /* Read the IV */
+   mdecrypt_cbc( td, ciphertext, blocksize);
+   
   /* decryption starts here */
    do
    {
@@ -1089,7 +1107,7 @@ switch (mode) {
         memmove(ciphertext_orig,ciphertext,blocksize); /* keep backup */
 
 
-        mdecrypt(td, ciphertext);
+        mdecrypt_cbc(td, ciphertext, blocksize);
 	
         memmove(ciphertext_old,ciphertext_orig,blocksize);
 
@@ -1120,6 +1138,102 @@ switch (mode) {
    }
    while(ch==0);
    break;
+
+ case ECB:
+  /* decryption starts here */
+   do
+   {
+
+    /* reads the first n bytes of the file and stores the to ciphertext */
+
+           i=fread( ciphertext, 1, blocksize, FROMF);
+
+           if (i < blocksize) {
+             if (ferror(FROMF)!=0) {
+               perror("fread"); 
+               return 1;
+             }
+             else { /* eof */
+               ch=1;
+               if (i==0) {fprintf(stderr,_("Incorrect file type\n")); return 1;}
+               memmove(&counter, ciphertext, sizeof(counter));
+               if (counter >= blocksize) {fprintf(stderr,_("Incorrect file type\n")); return 1;}
+             }
+           }
+
+         /* Decrypt the ciphertext */
+
+        memmove(ciphertext_orig,ciphertext,blocksize); /* keep backup */
+
+
+        mdecrypt_ecb(td, ciphertext,blocksize);
+	
+        memmove(ciphertext_old,ciphertext_orig,blocksize);
+
+
+/* Buffer isn't immediately written here. It waits a round to do
+ * so we can keep track of the end of file garbage and the IV at the begining
+ */
+
+		if (ch==1) {
+                  if (counter!=0) {
+		    if (fwrite(buffer,(int)counter,1,RTOF)==0) {perror("fwrite"); return 1;}
+  		     /* CRC32 */
+          	    if (bare_flag==FALSE) crc32(buffer, (int)counter);
+          	  }
+
+		}
+		else {
+		    if (start!=0) {
+   		      if (fwrite(buffer,blocksize,1,RTOF)==0) {perror("fwrite"); return 1;}
+                       /* CRC32 */
+                      if (bare_flag==FALSE) crc32(buffer, blocksize);
+
+   		    }
+                }
+
+         start++;
+         memmove(buffer,ciphertext,blocksize);  /* copy the plaintext to buffer */
+
+   }
+   while(ch==0);
+   break;
+   
+ case CFB:
+   end_mcrypt_ecb(td);
+   i=fread( ciphertext, 1, blocksize, FROMF); /* Read the IV */
+   td=init_mcrypt_cfb(algorithm,keyword,len,ciphertext); 
+
+  /* decryption starts here */
+
+   do
+   {
+
+    /* reads 1 byte of the file and stores the to ciphertext */
+
+           i=fread( ciphertext, 1, sizeof(ciphertext), FROMF);
+
+           if (i < 1) {
+             if (ferror(FROMF)!=0) {
+               perror("fread"); 
+               return 1;
+             }
+             ch=1;
+           }
+
+         /* Decrypt the ciphertext */
+
+        mdecrypt_cfb(td, ciphertext, i);
+	
+            if (ch!=1) {
+              if (fwrite(ciphertext,1, i,RTOF)==0) {perror("fwrite"); return 1;}
+                /* CRC32 */
+                if (bare_flag==FALSE) crc32(ciphertext, i);
+            }
+
+   }
+   while(ch==0);
+   break;   
 } /* switch */
 
 
@@ -1137,7 +1251,14 @@ switch (mode) {
 
 /* ready */
 
-  end_mcrypt(td);  
+  switch (mode) {
+    case CBC:
+      end_mcrypt_cbc(td);
+      break;
+    case ECB:
+      end_mcrypt_ecb(td);
+      break;
+  }
 
   free(ciphertext_old);
   free(buffer);
